package cd.casic.ci.process.process.service.target.impl;


import cd.casic.ci.process.dal.req.target.TargetManagerCreateReq;
import cd.casic.ci.process.dal.req.target.TargetManagerUpdateReq;
import cd.casic.ci.process.dal.req.target.TargetQueryReq;
import cd.casic.ci.process.dal.resp.target.TargetManagerResp;
import cd.casic.ci.process.process.converter.TargetConverter;
import cd.casic.ci.process.process.dal.pipeline.TargetManagerDao;
import cd.casic.ci.process.process.dal.pipeline.TargetVersionDao;
import cd.casic.ci.process.process.dataObject.base.BaseIdReq;
import cd.casic.ci.process.process.dataObject.target.TargetManager;
import cd.casic.ci.process.process.dataObject.target.TargetVersion;
import cd.casic.ci.process.process.service.target.TargetManagerService;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.framework.commons.util.object.BeanUtils;
import cd.casic.framework.commons.util.util.WebFrameworkUtils;
import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * @author HopeLi
 * @version v1.0
 * @ClassName TargetManagerServiceImpl
 * @Date: 2025/5/17 15:19
 * @Description:
 */
@Service
@Slf4j
public class TargetManagerServiceImpl extends ServiceImpl<TargetManagerDao, TargetManager> implements TargetManagerService {

    @Resource
    private TargetVersionServiceImpl targetVersionService;

    @Resource
    private TargetManagerDao targetManagerDao;

    @Resource
    private TargetVersionDao targetVersionDao;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createTarget(TargetManagerCreateReq req) {
        TargetManager targetManager = new TargetManager();
        BeanUtils.copyProperties(req, targetManager);
        targetManager.setCreator(String.valueOf(WebFrameworkUtils.getLoginUserId()));
        targetManager.setCreateTime(LocalDateTime.now());
        this.save(targetManager);

        //保存第一个版本
        TargetVersion targetVersion = new TargetVersion();
        BeanUtil.copyProperties(targetManager,targetVersion);
        targetVersion.setTargetId(targetManager.getId());
        targetVersion.setFilePath(req.getFilePath());
        targetVersion.setCreator(String.valueOf(WebFrameworkUtils.getLoginUserId()));
        targetVersion.setCreateTime(LocalDateTime.now());
        targetVersion.setFileSize(targetManager.getTotalSize());
        targetVersionService.save(targetVersion);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateTarget(TargetManagerUpdateReq req) {
        if (StringUtils.isEmpty(req.getId())){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"id不能为空");
        }

        TargetManager targetManager = targetManagerDao.selectById(req.getId());
        if (ObjectUtils.isEmpty(targetManager)){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"数据错误，请联系管理员");
        }

        BeanUtils.copyProperties(req,targetManager);
        targetManager.setUpdater(String.valueOf(WebFrameworkUtils.getLoginUserId()));
        targetManager.setUpdateTime(LocalDateTime.now());
        targetManagerDao.updateById(targetManager);

        //新增一条版本信息
        TargetVersion targetVersion = new TargetVersion();
        BeanUtil.copyProperties(targetManager,targetVersion);
        targetVersion.setTargetId(targetManager.getId());
        targetVersion.setFilePath(req.getFilePath());
        targetVersion.setCreator(String.valueOf(WebFrameworkUtils.getLoginUserId()));
        targetVersion.setCreateTime(LocalDateTime.now());
        targetVersion.setFileSize(targetManager.getTotalSize());
        targetVersionService.save(targetVersion);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteTarget(BaseIdReq req) {
        if (StringUtils.isEmpty(req.getId())){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"id不能为空");
        }

        TargetManager targetManager = targetManagerDao.selectById(req.getId());
        if (ObjectUtils.isEmpty(targetManager)){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"数据错误，请联系管理员");
        }
        targetManagerDao.deleteById(targetManager.getId());

        //删除对应的version版本
        List<TargetVersion> versionList = targetVersionService.selectListByManagerId(targetManager.getId());
        List<String> versionIdList = versionList.stream().map(TargetVersion::getId).toList();
        targetVersionDao.deleteByIds(versionIdList);
    }

    @Override
    public PageResult<TargetManagerResp> findTargetPage(TargetQueryReq query) {
        Page<TargetManagerResp> respPage = new Page<>();

        QueryWrapper<TargetManager> wrapper = new QueryWrapper<>();
        if (!ObjectUtils.isEmpty(query.getId())){
            wrapper.eq("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getIdList())){
            wrapper.in("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getTargetName())){
            wrapper.like("target_name",query.getTargetName());
        }
        if (!ObjectUtils.isEmpty(query.getCreator())){
            wrapper.eq("creator",query.getCreator());
        }

        Page<TargetManager> targetManagerPage = targetManagerDao.selectPage(new Page<>(query.getPageNo(), query.getPageSize()), wrapper);

        if (ObjectUtils.isEmpty(targetManagerPage)){
            return new PageResult<>();
        }

        List<TargetManagerResp> respList = TargetConverter.INSTANCE.toRespList(targetManagerPage.getRecords());

        respPage.setRecords(respList);
        respPage.setTotal(targetManagerPage.getTotal());
        respPage.setCurrent(targetManagerPage.getCurrent());
        respPage.setSize(targetManagerPage.getSize());
        PageResult<TargetManagerResp> pageResult = new PageResult<>(respList,targetManagerPage.getTotal());
        return pageResult;
    }

    @Override
    public List<TargetManagerResp> findTargetList(TargetQueryReq query) {
        QueryWrapper<TargetManager> wrapper = new QueryWrapper<>();
        if (!ObjectUtils.isEmpty(query.getId())){
            wrapper.eq("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getIdList())){
            wrapper.in("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getTargetName())){
            wrapper.like("target_name",query.getTargetName());
        }
        if (!ObjectUtils.isEmpty(query.getCreator())){
            wrapper.eq("creator",query.getCreator());
        }
        List<TargetManager> targetManagerList = targetManagerDao.selectList(wrapper);
        if (ObjectUtils.isEmpty(targetManagerList)){
            return new ArrayList<>(0);
        }

        List<TargetManagerResp> respList = TargetConverter.INSTANCE.toRespList(targetManagerList);

        return respList;
    }
}
